@@ -1820,7 +1820,7 @@ g_markup_parse_context_end_parse (GMarkupParseContext  *context,
  *
  * Since: 2.2
  */
-G_CONST_RETURN gchar *
+const gchar *
 g_markup_parse_context_get_element (GMarkupParseContext *context)
 {
   g_return_val_if_fail (context != NULL, NULL);
@@ -1851,7 +1851,7 @@ g_markup_parse_context_get_element (GMarkupParseContext *context)
  *
  * Since: 2.16
  */
-G_CONST_RETURN GSList *
+const GSList *
 g_markup_parse_context_get_element_stack (GMarkupParseContext *context)
 {
   g_return_val_if_fail (context != NULL, NULL);
