@@ -633,8 +633,10 @@ test_GDateTime_new_from_unix_utc (void)
 
   t = g_get_real_time ();
 
+#if 0
   dt = g_date_time_new_from_unix_utc (t);
   g_assert (dt == NULL);
+#endif
 
   t = t / 1e6;  /* oops, this was microseconds */
 
@@ -1105,6 +1107,22 @@ test_z (void)
   g_free (p);
 }
 
+static void
+test_refresh (void)
+{
+  GTimeZone *zone;
+
+  zone = g_time_zone_new (NULL);
+  g_assert (zone != NULL);
+  g_time_zone_unref (zone);
+
+  g_time_zone_refresh_local ();
+
+  zone = g_time_zone_new (NULL);
+  g_assert (zone != NULL);
+  g_time_zone_unref (zone);
+}
+
 gint
 main (gint   argc,
       gchar *argv[])
@@ -1151,6 +1169,7 @@ main (gint   argc,
   g_test_add_func ("/GDateTime/dst", test_GDateTime_dst);
   g_test_add_func ("/GDateTime/test_z", test_z);
   g_test_add_func ("/GDateTime/test-all-dates", test_all_dates);
+  g_test_add_func ("/GDateTime/refresh", test_refresh);
 
   return g_test_run ();
 }
